==> OpenCL-0.8.5.tgz: OpenCL-0.8.5/src/emac.cpp <==
/*************************************************
* EMAC Source File                               *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/emac.h>
#include <opencl/lookup.h>

namespace OpenCL {

/*************************************************
* Update an EMAC Calculation                     *
*************************************************/
void EMAC::add_data(const byte input[], u32bit length)
   {
   while(length)
      {
      u32bit xored = std::min(state.size() - position, length);
      xor_buf(state + position, input, xored);
      input += xored;
      length -= xored;
      position += xored;
      if(position == state.size())
         {
         f1->encrypt(state);
         position = 0;
         }
      }
   }

/*************************************************
* Finalize an EMAC Calculation                   *
*************************************************/
void EMAC::final_result(byte mac[])
   {
   for(u32bit j = position; j != state.size(); j++)
      state[j] ^= (byte)(state.size()-position);
   f1->encrypt(state);
   f2->encrypt(state, mac);
   state.clear();
   position = 0;
   }

/*************************************************
* EMAC Key Setup                                 *
*************************************************/
void EMAC::key(const byte key[], u32bit length)
   {
   SecureVector<byte> subkeys(2 * f1->MAXIMUM_KEYLENGTH);
   SecureVector<byte> block(f1->BLOCK_SIZE), counter(f1->BLOCK_SIZE);
   u32bit generated = 0;
   f1->set_key(key, length);
   while(generated < subkeys.size())
      {
      f1->encrypt(counter, block);
      subkeys.copy(generated, block, block.size());
      generated += block.size();
      for(s32bit j = counter.size() - 1; j >= 0; j--)
         if(++counter[j])
            break;
      }
   f1->set_key(subkeys, subkeys.size() / 2);
   f2->set_key(subkeys + subkeys.size() / 2, subkeys.size() / 2);
   }

/*************************************************
* Clear memory of sensitive data                 *
*************************************************/
void EMAC::clear() throw()
   {
   f1->clear();
   f2->clear();
   state.clear();
   position = 0;
   }

/*************************************************
* Return a clone of this object                  *
*************************************************/
MessageAuthenticationCode* EMAC::clone() const
   {
   return new EMAC(f1->clone());
   }

/*************************************************
* EMAC Constructor                               *
*************************************************/
EMAC::EMAC(const std::string& bc_name) :
   MessageAuthenticationCode(block_size_of(bc_name),
                             min_keylength_of(bc_name),
                             max_keylength_of(bc_name),
                             keylength_multiple_of(bc_name))
   {
   f1 = get_block_cipher(bc_name);
   f2 = f1->clone();
   state.create(OUTPUT_LENGTH);
   position = 0;
   }

/*************************************************
* EMAC Constructor                               *
*************************************************/
EMAC::EMAC(BlockCipher* c) :
   MessageAuthenticationCode(c->BLOCK_SIZE,
                             c->MINIMUM_KEYLENGTH,
                             c->MAXIMUM_KEYLENGTH,
                             c->KEYLENGTH_MULTIPLE)
   {
   f1 = c;
   f2 = c->clone();
   state.create(OUTPUT_LENGTH);
   position = 0;
   }

}

==> OpenCL-0.8.7.tgz: OpenCL-0.8.7/src/emac.cpp <==
/*************************************************
* EMAC Source File                               *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/emac.h>
#include <opencl/lookup.h>

namespace OpenCL {

/*************************************************
* Update an EMAC Calculation                     *
*************************************************/
void EMAC::add_data(const byte input[], u32bit length)
   {
   while(length)
      {
      u32bit xored = std::min(state.size() - position, length);
      xor_buf(state + position, input, xored);
      input += xored;
      length -= xored;
      position += xored;
      if(position == state.size())
         {
         f1->encrypt(state);
         position = 0;
         }
      }
   }

/*************************************************
* Finalize an EMAC Calculation                   *
*************************************************/
void EMAC::final_result(byte mac[])
   {
   for(u32bit j = position; j != state.size(); j++)
      state[j] ^= (byte)(state.size()-position);
   f1->encrypt(state);
   f2->encrypt(state, mac);
   state.clear();
   position = 0;
   }

/*************************************************
* EMAC Key Setup                                 *
*************************************************/
void EMAC::key(const byte key[], u32bit length)
   {
   SecureVector<byte> subkeys(2 * f1->MAXIMUM_KEYLENGTH);
   SecureVector<byte> block(f1->BLOCK_SIZE), counter(f1->BLOCK_SIZE);
   u32bit generated = 0;
   f1->set_key(key, length);
   while(generated < subkeys.size())
      {
      f1->encrypt(counter, block);
      subkeys.copy(generated, block, block.size());
      generated += block.size();
      for(s32bit j = counter.size() - 1; j >= 0; j--)
         if(++counter[j])
            break;
      }
   f1->set_key(subkeys, subkeys.size() / 2);
   f2->set_key(subkeys + subkeys.size() / 2, subkeys.size() / 2);
   }

/*************************************************
* Clear memory of sensitive data                 *
*************************************************/
void EMAC::clear() throw()
   {
   f1->clear();
   f2->clear();
   state.clear();
   position = 0;
   }

/*************************************************
* Return a clone of this object                  *
*************************************************/
MessageAuthenticationCode* EMAC::clone() const
   {
   return new EMAC(f1->name());
   }

/*************************************************
* EMAC Constructor                               *
*************************************************/
EMAC::EMAC(const std::string& bc_name) :
   MessageAuthenticationCode(block_size_of(bc_name),
                             min_keylength_of(bc_name),
                             max_keylength_of(bc_name),
                             keylength_multiple_of(bc_name))
   {
   f1 = get_block_cipher(bc_name);
   f2 = get_block_cipher(bc_name);
   state.create(OUTPUT_LENGTH);
   position = 0;
   }

}
